from active_bo_msgs.action import RLRollOut

import rclpy
from rclpy.node import Node
from rclpy.action import ActionServer

from active_bo_ros.ReinforcementLearning.ContinuousMountainCar import Continuous_MountainCarEnv

import numpy as np
import time

class RLActionServer(Node):
    def __init__(self):
        super().__init__('rl_action')
        self.action_server = ActionServer(self,
                                          RLRollOut,
                                          'reinforcement_learning_action',
                                          self.execute_callback)

        self.env = Continuous_MountainCarEnv(render_mode='rgb_array')
        self.distance_penalty = 0

    def execute_callback(self, action_handle):

        feedback_msg = RLRollOut.Feedback()

        reward = 0
        step_count = 0
        policy = action_handle.request.policy

        for i in range(len(policy)):
            action = policy[i]
            output = self.env.step(action)

            reward += output[1]
            done = output[2]
            step_count += 1

            rgb_array = output[5]
            rgb_shape = rgb_array.shape

            feedback_msg.height = rgb_shape[0]
            feedback_msg.width = rgb_shape[1]

            action_handle.publish_feedback(feedback_msg)

            if done:
                break

            distance = -(self.env.goal_position - output[0][0])
            reward += distance * self.distance_penalty

            time.sleep(0.01)

        action_handle.succeed()

        result = RLRollOut.Result()
        result.reward = reward
        result.final_step = step_count

        return result

def main(args=None):
    rclpy.init(args=args)

    rl_action_server = RLActionServer()

    rclpy.spin(rl_action_server)

if __name__ == '__main__':
    main()
